@@ -22,7 +22,7 @@ from Hellanzb.Logging import initLogging, stdinEchoOn
 from Hellanzb.PostProcessorUtil import defineMusicType
 from Hellanzb.Util import *
 
-__id__ = '$Id: Core.py 835 2006-09-11 06:18:24Z pjenvey $'
+__id__ = '$Id: Core.py 1053 2007-03-23 21:44:14Z pjenvey $'
 
 def findAndLoadConfig(optionalConfigFile = None):
     """ Find and load the configuration file """
@@ -35,7 +35,14 @@ def findAndLoadConfig(optionalConfigFile = None):
             sys.exit(1)
 
     # look for conf in this order: sys.prefix, ./, or ./etc/
-    confDirs = [ os.path.join(sys.prefix, 'etc'), os.path.join(os.getcwd(), 'etc'), os.getcwd() ]
+    confDirs = [os.path.join(sys.prefix, 'etc')]
+    try:
+        confDirs.append(os.path.join(os.getcwd(), 'etc'))
+        confDirs.append(os.getcwd())
+    except OSError, ose:
+        if ose.errno != 2:
+            raise
+        # OSError: [Errno 2] No such file or directory. cwd doesn't exist
 
     # hard coding preferred Darwin config file location, kind of lame. but I'd rather do
     # this then make an etc dir in os x's Python.framework directory
@@ -94,6 +101,16 @@ def loadConfig(fileName):
         else:
             Hellanzb.PAR2_CMD = assertIsExe([Hellanzb.PAR2_CMD])
 
+        if not hasattr(Hellanzb, 'MACBINCONV_CMD') or Hellanzb.MACBINCONV_CMD is None:
+            # macbinconv is optional when not explicitly specified in the conf
+            Hellanzb.MACBINCONV_CMD = None
+            try:
+                Hellanzb.MACBINCONV_CMD = assertIsExe(['macbinconv'])
+            except FatalError:
+                pass
+        else:
+            Hellanzb.MACBINCONV_CMD = assertIsExe([Hellanzb.MACBINCONV_CMD])
+
         if not hasattr(Hellanzb, 'SKIP_UNRAR') or Hellanzb.SKIP_UNRAR is None:
             Hellanzb.SKIP_UNRAR = False
 
@@ -101,6 +118,22 @@ def loadConfig(fileName):
         if not hasattr(Hellanzb, 'SMART_PAR'):
             Hellanzb.SMART_PAR = True
 
+        if not hasattr(Hellanzb, 'CATEGORIZE_DEST'):
+            Hellanzb.CATEGORIZE_DEST = True
+
+        if not hasattr(Hellanzb, 'NZB_ZIPS'):
+            Hellanzb.NZB_ZIPS = '.nzb.zip'
+        if not hasattr(Hellanzb, 'NZB_GZIPS'):
+            Hellanzb.NZB_GZIPS = '.nzb.gz'
+
+        if not hasattr(Hellanzb, 'DISABLE_COLORS'):
+            Hellanzb.DISABLE_COLORS = False
+
+        if not hasattr(Hellanzb, 'DISABLE_ANSI'):
+            Hellanzb.DISABLE_ANSI = False
+
+        Hellanzb.CACHE_LIMIT = unPrettyBytes(getattr(Hellanzb, 'CACHE_LIMIT', 0))
+
         if not hasattr(Hellanzb, 'OTHER_NZB_FILE_TYPES'):
             # By default, just match .nzb files in the queue dir
             Hellanzb.NZB_FILE_RE = re.compile(r'(?i)\.(nzb)$')
@@ -117,7 +150,8 @@ def loadConfig(fileName):
         for expandPath in ('PREFIX_DIR', 'QUEUE_DIR', 'DEST_DIR', 'POSTPONED_DIR',
                            'CURRENT_DIR', 'TEMP_DIR', 'PROCESSING_DIR', 'STATE_XML_FILE',
                            'WORKING_DIR', 'LOG_FILE', 'DEBUG_MODE',
-                           'UNRAR_CMD', 'PAR2_CMD', 'EXTERNAL_HANDLER_SCRIPT'):
+                           'UNRAR_CMD', 'PAR2_CMD', 'MACBINCONV_CMD',
+                           'EXTERNAL_HANDLER_SCRIPT'):
                 if hasattr(Hellanzb, expandPath):
                         thisDir = getattr(Hellanzb, expandPath)
                         if thisDir is not None:
@@ -245,15 +279,21 @@ def init(options = {}):
     # Whether or not this is a hellanzb download daemon process
     Hellanzb.isDaemon = False
 
+    # Whether or not we're currently downloading an NZB
+    Hellanzb.downloading = False
+
     # The name of the loaded config file
     Hellanzb.CONFIG_FILENAME = None
 
     # hostname we're running on
     Hellanzb.HOSTNAME = gethostname()
 
-    (sysname, nodename, release, version, machine) = os.uname()
-    # The OS in use
-    Hellanzb.SYSNAME = sysname
+    if isWindows():
+        Hellanzb.SYSNAME = None
+    else:
+        (sysname, nodename, release, version, machine) = os.uname()
+        # The OS in use
+        Hellanzb.SYSNAME = sysname
 
     # Only add anonymous NZB files placed in the QUEUE_DIR to the NZBQueue after this
     # number have seconds have passed since the files modification time
@@ -310,6 +350,19 @@ def init(options = {}):
             error('Config file option: Hellanzb.UMASK is not a valid integer')
             sys.exit(1)
 
+    if not hasattr(Hellanzb, 'LIBNOTIFY_NOTIFY'):
+        Hellanzb.LIBNOTIFY_NOTIFY = False
+    elif Hellanzb.LIBNOTIFY_NOTIFY:
+        try:
+            import pynotify
+        except ImportError:
+            error('Please install notify-python or disable Hellanzb.LIBNOTIFY_NOTIFY')
+            sys.exit(1)
+
+        if not pynotify.init('hellanzb'):
+            error('Cannot initialize libnotify')
+            sys.exit(1)
+
     if not hasattr(Hellanzb, 'GROWL_NOTIFY'):
         error('Required option not defined in config file: Hellanzb.GROWL_NOTIFY')
         sys.exit(1)
@@ -347,7 +400,7 @@ def shutdown(killPostProcessors = False, message = None):
         # here
         Topen.killAll()
 
-    if not Hellanzb.shutdownMessage:
+    if not getattr(Hellanzb, 'shutdownMessage', None):
         Hellanzb.shutdownMessage = message
     
     # stop the twisted reactor
@@ -370,16 +423,19 @@ def finishShutdown():
         # Remove the temporary files with the encoded data. Any errors causing hellanzb to
         # shut down prematurely (like can't bind to specific port -- maybe another
         # hellanzb is running?) should unset this var so this doesn't get called
-        rmtree(Hellanzb.DOWNLOAD_TEMP_DIR)
+        try:
+            rmtree(Hellanzb.DOWNLOAD_TEMP_DIR)
+        except OSError:
+            pass
     if hasattr(Hellanzb, 'DEQUEUED_NZBS_DIR'):
         rmtree(Hellanzb.DEQUEUED_NZBS_DIR)
 
     if Hellanzb.shutdownMessage:
         logShutdown(Hellanzb.shutdownMessage)
     
-def shutdownAndExit(returnCode = 0):
+def shutdownAndExit(returnCode = 0, message = None):
     """ Shutdown hellanzb's twisted reactor, AND call sys.exit """
-    shutdown(killPostProcessors = True)
+    shutdown(killPostProcessors = True, message = message)
 
     sys.exit(returnCode)
 
@@ -393,17 +449,11 @@ def marquee():
         options.append('daemonized')
     if Hellanzb.HAVE_C_YENC:
         options.append('C yenc module')
+    if Hellanzb.MACBINCONV_CMD is not None:
+        options.append('MacBinary')
 
     optionLen = len(options)
-    msg += ' ('
-
-    i = 0
-    for option in options:
-        msg += option
-        i += 1
-        if i < optionLen:
-            msg += ', '
-    msg += ')'
+    msg += ' (%s)' % ', '.join(options)
 
     info(msg)
     debug(msg)
@@ -443,8 +493,9 @@ def parseArgs():
     parser.add_option('-d', '--debug-file', type='string', dest='debugLogFile',
                       help='specify the debug log file (turns on debugging output/overwrites the ' + \
                       'Hellanzb.DEBUG_MODE config file setting)')
-    parser.add_option('-D', '--daemon', action='store_true', dest='daemonize',
-                      help='run hellanzb as a daemon process (fork and exit)')
+    if not isWindows():
+        parser.add_option('-D', '--daemon', action='store_true', dest='daemonize',
+                          help='run hellanzb as a daemon process (fork and exit)')
     #parser.add_option('-n', '--just-download-nzb', type='string', dest='justDownload',
     #                  help='download the specified nzb and exit the program (do not post process)')
     parser.add_option('-p', '--post-process-dir', type='string', dest='postProcessDir',
@@ -469,7 +520,7 @@ def processArgs(options, args):
     if not len(args) and not options.postProcessDir:
         Hellanzb.IS_DOWNLOADER = True
         
-        if options.daemonize:
+        if getattr(options, 'daemonize', False):
             # Run as a daemon process (fork)
             Hellanzb.DAEMONIZE = True
 
@@ -544,5 +595,5 @@ LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
-$Id: Core.py 835 2006-09-11 06:18:24Z pjenvey $
+$Id: Core.py 1053 2007-03-23 21:44:14Z pjenvey $
 """
