@@ -15,31 +15,20 @@ from Hellanzb.Logging import prettyException
 from Hellanzb.PostProcessorUtil import *
 from Hellanzb.Util import *
 
-__id__ = '$Id: PostProcessor.py 848 2006-10-21 20:39:41Z pjenvey $'
+__id__ = '$Id: PostProcessor.py 1030 2007-02-21 07:33:52Z pjenvey $'
 
 class PostProcessor(Thread):
     """ A post processor (formerly troll) instance runs in its own thread """
-    dirName = None
-    decompressionThreadPool = None
-    decompressorCondition = None
-    background = None
-    musicFiles = None
-    brokenFiles = None
-
-    failedLock = None
-    failedToProcesses = None
-    
-    msgId = None
-    nzbFile = None
 
+    # These attributes are routed to self.archive via __getattr__/__setattr__
     archiveAttrs = ('id', 'isParRecovery', 'rarPassword', 'deleteProcessed', 'skipUnrar',
-                    'toStateXML', 'msgid')
+                    'toStateXML', 'msgid', 'category')
 
     def __init__(self, archive, background = True, subDir = None):
         """ Ensure sanity of this instance before starting """
         # The archive to post process
         self.archive = archive
-    
+        
         # DirName is a hack printing out the correct directory name when running nested
         # post processors on sub directories
         if subDir:
@@ -49,10 +38,12 @@ class PostProcessor(Thread):
             self.subDir = subDir
             self.dirName = DirName(pathjoin(archive.archiveDir, self.subDir))
             self.dirName.parentDir = archive.archiveDir
+            self.archiveName = archiveName(self.dirName.parentDir)
         else:
             self.isSubDir = False
             self.dirName = DirName(archive.archiveDir)
             self.archive.postProcessor = self
+            self.archiveName = archiveName(self.dirName)
 
         self.nzbFileName = None
         if self.isNZBArchive():
@@ -75,10 +66,16 @@ class PostProcessor(Thread):
         self.decompressorLock = RLock()
         self.decompressorCondition = Condition(self.decompressorLock)
 
+        self.musicFiles = []
+        self.brokenFiles = []
         self.movedSamples = []
-        
+
         self.startTime = None
 
+        # Failed decompress threads put their file names in this list
+        self.failedLock = Lock()
+        self.failedToProcesses = []
+        
         # Whether or not this PostProcessor's Topen processes were explicitly kill()'ed
         self.killed = False
 
@@ -92,7 +89,9 @@ class PostProcessor(Thread):
 
     def __getattr__(self, name):
         """ Forward specific attribute lookups to the Archive object """
-        if name in self.archiveAttrs:
+        if name == 'category' and not Hellanzb.CATEGORIZE_DEST:
+            return ''
+        elif name in self.archiveAttrs:
             return getattr(self.archive, name)
         raise AttributeError, name
 
@@ -144,6 +143,9 @@ class PostProcessor(Thread):
             if not self.killed and not self.isSubDir and self.background:
                 Hellanzb.writeStateXML()
 
+        # When a Post Processor fails, we end up moving the destDir here
+        self.moveDestDir() 
+
         # FIXME: This isn't the best place to GC. The best place would be when a download
         # is finished (idle NZBLeecher) but with smartpar, finding an idle NZBLeecher is
         # tricky
@@ -157,9 +159,6 @@ class PostProcessor(Thread):
                 self.callback()
                 return
             
-        # When a Post Processor fails, we end up moving the destDir here
-        self.moveDestDir() 
-
         if not self.background and not self.isSubDir:
             # We're not running in the background of a downloader -- we're post processing
             # and then immeidately exiting (-Lp)
@@ -180,8 +179,16 @@ class PostProcessor(Thread):
                 os.remove(self.dirName)
 
             elif os.path.isdir(self.dirName):
+                if not os.path.isdir(os.path.join(Hellanzb.DEST_DIR, self.category)):
+                    try:
+                        os.makedirs(os.path.join(Hellanzb.DEST_DIR, self.category))
+                    except OSError, ose:
+                        raise FatalError('Unable to create directory for category: ' + \
+                                os.path.join(Hellanzb.DEST_DIR, self.category)  + \
+                                ' error: ' + str(ose))                
                 # A dir in the processing dir, move it to DEST
-                newdir = os.path.join(Hellanzb.DEST_DIR, os.path.basename(self.dirName))
+                newdir = os.path.join(Hellanzb.DEST_DIR, self.category,
+                                      os.path.basename(self.dirName))
                 hellaRename(newdir)
                 move(self.dirName, newdir)
                 
@@ -215,6 +222,7 @@ class PostProcessor(Thread):
             # REACTOR STOPPED IF NOT BACKGROUND/SUBIDR
             if self.background:
                 logStateXML(debug)
+            category = self.category
             self.stop()
 
             # Propagate up to the original Post Processor
@@ -239,7 +247,7 @@ class PostProcessor(Thread):
 
             e = time.time() - self.startTime 
             dispatchExternalHandler(ERROR, archiveName=archive,
-                                    destDir=os.path.join(Hellanzb.DEST_DIR, archive),
+                                    destDir=os.path.join(Hellanzb.DEST_DIR, category, archive),
                                     elapsedTime=prettyElapsed(e),
                                     parMessage='A problem occurred: %s' % pe)
 
@@ -270,11 +278,15 @@ class PostProcessor(Thread):
             return
         
         # Determine the music files to decompress
-        self.musicFiles = []
+        musicTypes = []
         for file in os.listdir(self.dirName):
             absPath = os.path.join(self.dirName, file)
-            if os.path.isfile(absPath) and getMusicType(file) and getMusicType(file).shouldDecompress():
+            musicType = getMusicType(file)
+            if os.path.isfile(absPath) and musicType and musicType.shouldDecompress():
                 self.musicFiles.append(absPath)
+                if musicType not in musicTypes:
+                    musicTypes.append(musicType)
+        musicTypes.sort()
     
         if len(self.musicFiles) == 0:
             return
@@ -285,19 +297,19 @@ class PostProcessor(Thread):
         
         filesTxt = 'file'
         threadsTxt = 'thread'
+        musicTypesPrefix = 'format'
         if len(self.musicFiles) != 1:
             filesTxt += 's'
         if threadCount != 1:
             threadsTxt += 's'
+        if len(musicTypes) > 1:
+            musicTypesPrefix += 's'
 
         fileCount = len(self.musicFiles)
-        info(archiveName(self.dirName) + ': Decompressing ' + str(fileCount) + \
-             ' ' + filesTxt + ' via ' + str(threadCount) + ' ' + threadsTxt + '..')
-
-        # Failed decompress threads put their file names in this list
-        self.failedToProcesses = []
-        self.failedLock = Lock()
-
+        musicTypesTxt = ', '.join([musicType.extension for musicType in musicTypes])
+        info('%s: Decompressing %i %s (%s: %s) via %i %s..' % \
+             (archiveName(self.dirName), fileCount, filesTxt, musicTypesPrefix, musicTypesTxt,
+              threadCount, threadsTxt))
         start = time.time()
         # Maintain a pool of threads of the specified size until we've exhausted the
         # musicFiles list
@@ -348,12 +360,6 @@ class PostProcessor(Thread):
         # Move other cruft out of the way
         deleteDuplicates(self.dirName)
         
-        if self.nzbFile is not None:
-            if os.path.isfile(os.path.join(self.dirName, self.nzbFile)) and \
-                    os.access(os.path.join(self.dirName, self.nzbFile), os.R_OK):
-                move(os.path.join(self.dirName, self.nzbFile),
-                     os.path.join(self.dirName, Hellanzb.PROCESSED_SUBDIR, self.nzbFile))
-
         # Move out anything else that's broken, a dupe or tagged as
         # not required
         for file in self.brokenFiles:
@@ -402,19 +408,24 @@ class PostProcessor(Thread):
             parMessage = ''
             if not handledPars:
                 parMessage = ' (No Pars)'
-                
+            totalTime = ''
+            if self.isNZBArchive():
+                totalTime = ' (total: %s)' % \
+                    prettyElapsed(e + self.archive.downloadAndDecodeTime)
+
+            elapsed = prettyElapsed(e)
             archive = archiveName(self.dirName)
-            info('%s: Finished processing (took: %s)%s' % (archive,
-                                                           prettyElapsed(e), parMessage))
+            info('%s: Finished processing (took: %s)%s%s' % (archive, 
+                                                           elapsed, totalTime, parMessage))
             dispatchExternalHandler(SUCCESS, archiveName=archive,
-                                    destDir=os.path.join(Hellanzb.DEST_DIR, archive),
+                                    destDir=os.path.join(Hellanzb.DEST_DIR, self.category, archive),
                                     elapsedTime=prettyElapsed(e),
                                     parMessage=parMessage)
 
             if parMessage != '':
                 parMessage = '\n' + parMessage
-            growlNotify('Archive Success', 'hellanzb Done Processing' + parMessage + ':',
-                        archive, True)
+            notify('Archive Success', 'hellanzb Done Processing %s:' % parMessage,
+                        '%s\ntook: %s%s' % (archive, elapsed, totalTime), True)
                        #self.background)
         # FIXME: could unsticky the message if we're running hellanzb.py -p
         # and preferably if the post processing took say over 30 seconds
@@ -453,22 +464,13 @@ class PostProcessor(Thread):
             raise FatalError('Unable to create processed dir, a non dir already exists there: ' + \
                              processedDir)
     
-        # First, find broken files, in prep for repair. Grab the msg id and nzb
-        # file names while we're at it
-        self.brokenFiles = []
+        # First, find broken files, in prep for repair. Grab the msg id while we're at it
         files = os.listdir(self.dirName)
         for file in files:
             absoluteFile = os.path.join(self.dirName, file)
-            if os.path.isfile(absoluteFile):
-                if file.endswith('_broken'):
-                    # Keep track of the broken files
-                    self.brokenFiles.append(absoluteFile)
-                    
-                elif len(file) > 7 and file[0:len('.msgid_')] == '.msgid_':
-                    self.msgId = file[len('.msgid_'):]
-    
-                elif len(file) > 3 and file.find('.') > -1 and getFileExtension(file).lower() == 'nzb':
-                    self.nzbFile = file
+            if os.path.isfile(absoluteFile) and file.endswith('_broken'):
+                # Keep track of the broken files
+                self.brokenFiles.append(absoluteFile)
     
         # If there are required broken files and we lack pars, punt
         if len(self.brokenFiles) > 0 and containsRequiredFiles(self.brokenFiles) and \
@@ -492,6 +494,7 @@ class PostProcessor(Thread):
                 processPars(self, needAssembly)
             except ParExpectsUnsplitFiles:
                 info(archiveName(self.dirName) + ': This archive requires assembly before running par2')
+                decodeMacBin(self)
                 assembleSplitFiles(self.dirName, needAssembly)
                 try:
                     processPars(self, None)
@@ -510,6 +513,7 @@ class PostProcessor(Thread):
             cleanHellanzbTmpFiles(self.dirName)
         
         # Rars may need assembly before unraring
+        decodeMacBin(self)
         assembleSplitFiles(self.dirName, findSplitFiles(self.dirName))
 
         if not Hellanzb.SKIP_UNRAR and dirHasRars(self.dirName):
@@ -522,6 +526,7 @@ class PostProcessor(Thread):
             self.processMusic()
 
         # Assemble split up files (that were just unrared)
+        decodeMacBin(self)
         assembleSplitFiles(self.dirName, findSplitFiles(self.dirName))
 
         # FIXME: do we need to gc.collect() after post processing a lot of data?
@@ -607,5 +612,5 @@ LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
 
-$Id: PostProcessor.py 848 2006-10-21 20:39:41Z pjenvey $
+$Id: PostProcessor.py 1030 2007-02-21 07:33:52Z pjenvey $
 """
